/**
 * @file ChangeVT.cpp
 * @date 2010/04/24 新規作成
 * @author
 *
 * VT変更クラス
 */
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/vt.h>
#include <sys/kd.h>
#include <linux/fb.h>
#include <sstream>

#include "ChangeVT.h"

/**
 * コンストラクタ
 */
ChangeVT::ChangeVT(){
}

/**
 * consoleとframebufferの結びつけ取得
 * @param fd フレームバッファ
 * @param vt VT番号
 * @return framebufferディスクリプタ
 */
int ChangeVT::getFb(int fd, int vt)
{
	struct fb_con2fbmap c2m;
	c2m.console = vt;
	if(ioctl(fd, FBIOGET_CON2FBMAP, &c2m)){
		return 0;
	}
	return c2m.framebuffer;
}

/**
 * consoleとframebufferの結びつけ
 * @param fd フレームバッファ
 * @param vt VT番号
 * @param fb 元のフレームバッファ
 * @return 処理結果 OK/NG
 */
ReturnValue ChangeVT::setFb(int fd, int vt, int fb)
{
	struct fb_con2fbmap c2m;
	struct stat         sbf;
	if( fstat(fd, &sbf) ){
		return NG;
	}
	if( fb >= 0 ){
		c2m.framebuffer = fb;
	}else{
		c2m.framebuffer = (sbf.st_rdev & 0xFF) >> 5;
	}
	c2m.console = vt;
	if( ioctl(fd, FBIOPUT_CON2FBMAP, &c2m) == -1 ){
		return NG;
	}
	return OK;
}

/**
 * VT変更
 * @return 処理結果 OK/NG
 */
ReturnValue ChangeVT::change()
{
	int fb_fd = open("/dev/fb0", O_RDWR);
	if( fb_fd == -1 ){
		return NG;
	}
	m_OldFb = getFb(fb_fd, m_VTNum);
	int con_fd = open("/dev/console", O_RDWR | O_NOCTTY);
	if( con_fd == -1 ){
		return NG;
	}
	struct vt_stat vs;
	if( ioctl(con_fd, VT_GETSTATE, &vs) == -1 ){
		return NG;
	}
	m_OldVT = vs.v_active;
	if( ioctl(con_fd, VT_OPENQRY, &m_VTNum) == -1 ){
		return NG;
	}
	if( setFb(fb_fd, m_VTNum, -1) == NG ){
		return NG;
	}
	close(fb_fd);
	if( ioctl(con_fd, VT_ACTIVATE, m_VTNum) == -1 ){
		return NG;
	}
	if( ioctl(con_fd, VT_WAITACTIVE, m_VTNum) == -1 ){
		return NG;
	}
	close(con_fd);
	std::stringstream tty_string;
	tty_string << "/dev/tty";
	tty_string << std::dec << m_VTNum;
	m_VTFd = open(tty_string.str().c_str(), O_RDWR | O_NOCTTY);
	if( m_VTFd == -1 ){
		return NG;
	}
	if( ioctl( m_VTFd, KDSETMODE, KD_GRAPHICS ) == -1 ){
		return NG;
	}
	return OK;
}

/**
 * VT戻し
 * @return 処理結果 OK/NG
 */
ReturnValue ChangeVT::cleanup()
{
	if( ioctl( m_VTFd, KDSETMODE, KD_TEXT ) == -1 ){
		return NG;
	}
	close(m_VTFd);
	int con_fd = open("/dev/console", O_RDWR | O_NOCTTY);
	if( con_fd == -1 ){
		return NG;
	}
	if( ioctl(con_fd, VT_ACTIVATE, m_OldVT) == -1 ){
		return NG;
	}
	if( ioctl(con_fd, VT_WAITACTIVE, m_OldVT) == -1 ){
		return NG;
	}
	if( ioctl( con_fd, VT_DISALLOCATE, m_VTNum ) == -1 ){
		return NG;
	}
	close(con_fd);
	int fb_fd = open("/dev/fb0", O_RDWR);
	if( fb_fd == -1 ){
		return NG;
	}
	if( setFb(fb_fd, m_VTNum, m_OldFb) == NG ){
		return NG;
	}
	close(fb_fd);
	return OK;
}

