/**
 * @file ParseLine.cpp
 * @date 2010/04/10 新規作成
 * @author
 *
 * iniファイル１行分構文解析クラス
 */
#include <iostream>
#include "Define.h"
#include "TrimSpace.h"
#include "ParseLine.h"

/**
 * コンストラクタ
 */
ParseLine::ParseLine(){
}

/**
 * セクション解析
 * @param buffer 読込み文字列
 * @return 処理結果 OK/NG
 */
IniLine ParseLine::parseSection(std::string buffer){
	TrimSpace trim_space;
	buffer = trim_space.trimEnds(buffer);
	if( buffer.at(0) != '[' ){
		return INVALID;
	}else{
		buffer.erase(0, 1);
	}
	if( buffer.at(buffer.size() - 1) != ']' ){
		return INVALID;
	}else{
		buffer.erase(buffer.size() - 1);
	}
	/* 禁則文字 */
	for(std::string::size_type i = 0; i < buffer.size(); i++){
		if( (buffer.at(i) >= '0') && (buffer.at(i) <= '9') ){
			continue;
		}else if( (buffer.at(i) >= 'a') && (buffer.at(i) <= 'z') ){
			continue;
		}else if( (buffer.at(i) >= 'A') && (buffer.at(i) <= 'Z') ){
			continue;
		}else if( buffer.at(i) == '_' ){
			continue;
		}else{
			return INVALID;
		}
	}
	return SECTION;
}

/**
 * パラメータ解析
 * @param buffer 読込み文字列
 * @return 処理結果 OK/NG
 */
IniLine ParseLine::parseParameter(std::string buffer){
	TrimSpace trim_space;
	std::string::size_type index = buffer.find('=', 0);
	std::string name = trim_space.trimEnds(buffer.substr(0, index));
	std::string value = trim_space.trimEnds(buffer.substr(index + 1));
	if( name.size() == 0 ){
		return INVALID;
	}
	/* 禁則文字 */
	std::string::size_type i;
	for(i = 0; i < name.size(); i++){
		if( (name.at(i) >= '0') && (name.at(i) <= '9') ){
			continue;
		}else if( (name.at(i) >= 'a') && (name.at(i) <= 'z') ){
			continue;
		}else if( (name.at(i) >= 'A') && (name.at(i) <= 'Z') ){
			continue;
		}else if( name.at(i) == '_' ){
			continue;
		}else{
			return INVALID;
		}
	}
	return PARAMETER;
}

/**
 * 構文解析
 * @param buffer 一行読込み文字列
 * @return 処理結果 OK/NG
 */
IniLine ParseLine::parse(std::string buffer){
	if( buffer.size() == 0 ){
		return NULL_LINE;
	}
	for(std::string::iterator iterator = buffer.begin(); iterator != buffer.end(); iterator++){
		if( *iterator == ' ' ){
			continue;
		}else if( *iterator == '\t' ){
			continue;
		}else if( *iterator == ';' ){
			return COMMENT;
		}else if( *iterator == '#' ){
			return COMMENT;
		}else if( *iterator == '[' ){
			return parseSection(buffer);
		}else{
			if( buffer.find('=') != std::string::npos ){
				return parseParameter(buffer);
			}else{
				return INVALID;
			}
		}
	}
	return INVALID;
}

