/**
 * @file ReadIniFile.cpp
 * @date 2010/04/10 新規作成
 * @author
 *
 * iniファイル読込みクラス
 */
#include <algorithm>
#include <fstream>
#include "Define.h"
#include "TrimSpace.h"
#include "ParseLine.h"
#include "ReadIniFile.h"

/**
 * コンストラクタ
 */
ReadIniFile::ReadIniFile(){
}

/**
 * セクション文字列整形
 * @param ini_file INIファイル型クラス
 * @param temporary セクション文字列
 * @return 処理結果 OK/NG
 */
ReturnValue ReadIniFile::setSection(TIniFile& ini_file, std::string temporary){
	/* まず空白を削除 */
	std::string::iterator iterator = std::remove(temporary.begin(), temporary.end(), ' ');
	temporary.erase(iterator, temporary.end());

	temporary.erase(temporary.find('[', 0), 1);
	temporary.erase(temporary.find(']', 0), 1);
	struct Section section;
	section.sectionName = temporary;
	ini_file.getData().push_back(section);
	return OK;
}

/**
 * パラメータ文字列整形
 * @param ini_file INIファイル型クラス
 * @param temporary パラメータ文字列
 * @return 処理結果 OK/NG
 */
ReturnValue ReadIniFile::setParameter(TIniFile& ini_file, std::string temporary){
	std::string::size_type index;
	index = temporary.find('=', 0);
	struct Parameter parameter;
	TrimSpace trim_space;
	parameter.name = trim_space.trimEnds(temporary.substr(0, index));
	parameter.value = trim_space.trimEnds(temporary.substr(index + 1));
	ini_file.getData().at(ini_file.getData().size() - 1).parameter.push_back(parameter);
	return OK;
}

/**
 * ファイル読込み
 * @param path 読み込みファイルパス
 * @return 
 */
TIniFile ReadIniFile::read(std::string path){
	TIniFile	ini_file;
	std::ifstream input(path.c_str());
	char buffer[256];
	ReturnValue return_value = NG;
	ParseLine parse_line;
	
	if( input != false ){
		while( input.eof() != true ){
			input.getline(buffer, sizeof(buffer));
			ParseLine parse_line;
			switch( parse_line.parse(buffer) ){
			case COMMENT :
				return_value = OK;
				break;
			case SECTION :
				return_value = setSection(ini_file, buffer);
				break;
			case PARAMETER :
				return_value = setParameter(ini_file, buffer);
				break;
			case NULL_LINE :
				return_value = OK;
				break;
			case INVALID :
			default :
				return_value = NG;
				break;
			}
			if( return_value == NG ){
				ini_file.getData().clear();
				break;
			}
		}
		input.close();
	}
	return ini_file;
}


